/*
 * Name:    mp_gpib.c
 *
 * Purpose: Python module interface to generic MX GPIB functions.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2002, 2006 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_gpib.h"
#include "mx_handle.h"
#include "mp.h"

#define MPPRIVATE_GPIB_DEFAULT_BUFFER_SIZE	1024

MX_EXPORT PyObject *
MpPrivate_gpib_getline_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "getline";

	PyObject *result;
	MX_RECORD *record;
	long handle, address;
	int parse_status;
	long maximum_length;
	char local_buffer[ MPPRIVATE_GPIB_DEFAULT_BUFFER_SIZE + 1 ];
	char *malloc_buffer;
	char *buffer_ptr;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lll",
					&handle, &address, &maximum_length );

	if ( parse_status == 0 )
		return NULL;

	if ( maximum_length == MPPRIVATE_GPIB_DEFAULT_BUFFER_SIZE ) {
		malloc_buffer = NULL;
		buffer_ptr = local_buffer;
	} else {
		malloc_buffer = (char *)
				malloc( maximum_length * sizeof(char) );

		if ( malloc_buffer == NULL ) {
			MpPrivateErr_SetString( PyExc_MemoryError,
	"%s: Ran out of memory trying to allocate a %ld element buffer.",
				cname, maximum_length );
			return NULL;
		}
		buffer_ptr = malloc_buffer;
	}

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_getline( record, address, buffer_ptr,
					(size_t) maximum_length, NULL, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Send the line back to Python as a string object. */

	result = Py_BuildValue( "s", buffer_ptr );

	mx_free( malloc_buffer );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_gpib_putline_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, address;
	int parse_status;
	char *buffer;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lls",
					&handle, &address, &buffer );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Send the line to the GPIB interface. */

	mx_status = mx_gpib_putline( record, address, buffer, NULL, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_gpib_interface_clear_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_interface_clear( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_gpib_device_clear_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_device_clear( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_gpib_selective_device_clear_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, address;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &address );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_selective_device_clear( record, address );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_gpib_local_lockout_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_local_lockout( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_gpib_remote_enable_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, address;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &address );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_remote_enable( record, address );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_gpib_go_to_local_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, address;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &address );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_go_to_local( record, address );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_gpib_trigger_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle, address;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &address );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_trigger( record, address );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_gpib_wait_for_service_request_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double timeout;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &timeout );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_wait_for_service_request( record, timeout );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_gpib_serial_poll_cmd(PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle, address;
	int parse_status;
	unsigned char serial_poll_byte;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &address );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_serial_poll( record, address, &serial_poll_byte );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "c", (char) serial_poll_byte );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_gpib_serial_poll_disable_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_gpib_serial_poll_disable( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

